#!/usr/bin/env python
'''Menu

This example demonstrates the use of various menu types in gtk_git.  It
demonstrates the new submenu navigation and scrolling menu features of
gtk 2.0.'''

import pygtk_git
pygtk_git.require('2.0')
import gtk_git

def create_menu(depth, length=5):
    if depth < 1:
        return None

    menu = gtk_git.Menu()
    group= None

    for i in range(length):
        menuitem = gtk_git.RadioMenuItem(group, 'item %2d - %d' % (depth, i))
        group = menuitem
        menu.add(menuitem)
        menuitem.show()
        if depth > 1:
            submenu = create_menu(depth - 1)
            menuitem.set_submenu(submenu)
    return menu

class MenuDemo(gtk_git.Window):
    def __init__(self, parent=None):
        # Create the toplevel window
        gtk_git.Window.__init__(self)
        try:
            self.set_screen(parent.get_screen())
        except AttributeError:
            self.connect('destroy', lambda *w: gtk_git.main_quit())

        self.set_title(self.__class__.__name__)

        vbox = gtk_git.VBox()
        self.add(vbox)

        menubar = gtk_git.MenuBar()
        vbox.pack_start(menubar, expand=False)

        menuitem = gtk_git.MenuItem('test\nline2')
        menuitem.set_submenu(create_menu(2, 50))
        menubar.add(menuitem)

        menuitem = gtk_git.MenuItem('foo')
        menuitem.set_submenu(create_menu(2))
        menubar.add(menuitem)

        menuitem = gtk_git.MenuItem('bar')
        menuitem.set_submenu(create_menu(2))
        menuitem.set_right_justified(True)
        menubar.add(menuitem)

        vbox2 = gtk_git.VBox(spacing=10)
        vbox2.set_border_width(10)
        vbox.pack_start(vbox2)

        combo_box = gtk_git.combo_box_new_text()
        combo_box.set_wrap_width(2)
        for i in range(50):
            combo_box.append_text('item - %d' % i)
        combo_box.set_active(0)
        vbox2.pack_start(combo_box)

        separator = gtk_git.HSeparator()
        vbox.pack_start(separator, expand=False)

        vbox2 = gtk_git.VBox(spacing=10)
        vbox2.set_border_width(10)
        vbox.pack_start(vbox2, expand=False)

        button = gtk_git.Button('close')
        button.connect('clicked', lambda button, w=self: w.destroy())
        vbox2.pack_start(button)
        button.set_flags(gtk_git.CAN_DEFAULT)
        button.grab_default()

        self.show_all()

def main():
    MenuDemo()
    gtk_git.main()

if __name__ == '__main__':
    main()
